  local addon, ns = ...
  local cfg = CreateFrame("Frame")

  -----------------------------
  -- MEDIA
  -----------------------------
  local MediaPath = "Interface\\Addons\\oUF_mono\\media\\"
  cfg.statusbar_texture = MediaPath.."statusbar"
  cfg.auratex = MediaPath.."iconborder" 
  cfg.font = MediaPath.."font.ttf"
  cfg.backdrop_texture = MediaPath.."backdrop"
  cfg.backdrop_edge_texture = MediaPath.."backdrop_edge"
  
  -----------------------------
  -- CONFIG
  -----------------------------

  --config variables
  cfg.showtot = true
  cfg.showpet = true
  cfg.showfocus = true
  cfg.showparty = true
  cfg.showpartypet = true
  cfg.showboss = true
  cfg.showarena = true
  cfg.showPortraits = false
  cfg.useSmooth = false
  cfg.useClassColorOnFullHealth = true
  cfg.allow_frame_movement = false -- allow alt+shift dragging for player, target, focus and pet frames
  cfg.frames_locked = true 
  cfg.interruptcb = {1, 0.49, 0} --color setting for uninterruptable casts
  cfg.stylebuffs = false -- replacing default buff frame
    cfg.BUFFpos = {"TOPRIGHT", "UIParent", "TOPRIGHT", -160, -20} -- buffs position
    cfg.DEBUFFpos = {"TOPRIGHT", "UIParent", "TOPRIGHT", -160, -125} -- debuffs position
    cfg.BFIsize = 32 -- buff frame icon size
  cfg.playerauras = false -- small aura frame for player
  cfg.auratimers = false -- aura timers
    cfg.ATIconSizeThreshold = 25 -- how big some icon should be to display the custom timer
    cfg.ATSize = 15 -- aura timer font size
  cfg.ArenaAuraTracker = true -- special tracker will be displayed @arena frames instead of normal buffs/debuffs
    cfg.AATTimerSize = 15 -- Arena AuraTracker timer font size
  cfg.showfaketarget = false -- fake target bars that spawn if you don't have anything targeted
  cfg.RMalpha = 0.6 -- raid mark alpha
  cfg.RMsize = 16 -- raid mark size
  cfg.focusCBuserplaced = false -- false to lock focus cast bar to the focus frame
    cfg.focusCBposition = {"CENTER",UIParent,"BOTTOM",10,410} -- focus cb position
    cfg.focusCBwidth = 280 -- focus cb width
    cfg.focusCBheight = 15 -- focus cb height

  -- Frames position
  cfg.Ppos = {"TOP","UIParent","BOTTOM", -235, 272} -- player
  cfg.Tpos = {"TOP","UIParent","BOTTOM", 235, 272} -- target
  cfg.PEpos = {"TOPLEFT", "oUF_monoPlayerFrame", "BOTTOMLEFT", 0, -40} -- pet
  cfg.TTpos = {"TOPRIGHT", "oUF_monoTargetFrame", "BOTTOMRIGHT", 0, -40} -- ToT
  cfg.Fpos = {"TOPLEFT", "oUF_monoTargetFrame", "TOPRIGHT", 3, 0} -- focus
  cfg.PApos = {"BOTTOMLEFT", "UIParent", "BOTTOMLEFT", 95, 352} -- party
    cfg.PAspacing = 40 -- spacing between party units
  cfg.ARpos = {"BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", -117, 392} -- arena
  cfg.BOpos = {"BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", -117, 392} -- boss
    cfg.ABspacing = 56 -- spacing between arena and boss units
  
  -- Size and scale
  cfg.Pwidth = 232 -- Player frame
  cfg.Pheight = 25
  cfg.Pscale = 1
  
  cfg.Twidth = 232 -- Target frame
  cfg.Theight = 25
  cfg.Tscale = 1
  
  cfg.PTTwidth = 133 -- Pet and ToT frames
  cfg.PTTheight = 20
  cfg.PTTscale = 0.9
  
  cfg.Fwidth = 120 -- Focus frame
  cfg.Fheight = 25
  cfg.Fscale = 1
  
  cfg.PABwidth = 196 -- Party (+partypet), Arena (+arenatargets) and Boss frames
  cfg.PABheight = 22
  cfg.PABscale = 0.9
  
  --raid config
  cfg.showraid = true
  cfg.aurafont = MediaPath.."auras.ttf"
  cfg.debuffborder = MediaPath.."iconborder"
  cfg.highlightTex = "Interface\\Buttons\\WHITE8x8"
  cfg.symbols = cfg.font
  cfg.width = 59 -- unit width
  cfg.height = 27 -- unit height
  cfg.spacing = 4 -- spacing between units
  cfg.namelength = 4 -- number of letters to display
  cfg.fontsize = 12
  cfg.iconsize = 10 -- informative icon size (aka RL,ML,A etc.)
  cfg.symbolsize = 11 -- bottom right corner counter size
  cfg.indicatorsize = 5 -- square indicator size
  cfg.debuffsize = 11 -- debuff icon size
  cfg.focusHLcol = {.8, .8, .2, .7} -- focus border color
  cfg.orientation = "HORIZONTAL" 
  cfg.pos = {"TOPLEFT", "UIParent", "BOTTOM", -156, 177} -- raid frame (group 1-5) position
  cfg.MTframes = true
    cfg.MTpos = {"BOTTOMLEFT", "UIParent", "BOTTOMRIGHT", -163, 233} -- MTs frame position
    cfg.MTsize = 1.5 -- MT size relatively to unit size
  cfg.frequent = true -- Enhances update rate for indicators (can be cpu intensive)
  cfg.indicators = true
  cfg.LeaderIcons = true
  cfg.RaidMark = true
  cfg.RCheckIcon = true
  cfg.rescomm = false
    cfg.rescommalpha = 0.5
  cfg.soloON = true -- show player frame if not in group
  cfg.partyON = true -- show party as 5 men raid group
  cfg.raid5ON = true -- show raid frame for 5 (or less) men raid group
    cfg.lfdIcons = true 
  cfg.powerbar = true
    cfg.powerbarsize = 0.09 -- power bar thickness relatively to unit size

  -- HANDOVER
  ns.cfg = cfg
