  local addon, ns = ...
  local cfg = ns.cfg
  local lib = ns.lib

  -----------------------------
  -- STYLE FUNCTIONS
  -----------------------------
  local function genStyle(self)
    lib.init(self)
    lib.gen_hpbar(self)
    lib.gen_hpstrings(self)
    lib.gen_ppbar(self)
    lib.gen_highlight(self)
    lib.gen_RaidMark(self)
  end  

  --the player style
  local function CreatePlayerStyle(self)
    self.width = cfg.Pwidth
    self.height = cfg.Pheight
    self.scale = cfg.Pscale
    self.mystyle = "player"
    lib.moveme(self)
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.Smooth = cfg.useSmooth
    if (not cfg.useClassColorOnFullHealth) and cfg.useSmooth then
      self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
      self.Health.colorSmooth = self.Health.Smooth
    end
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.Smooth = cfg.useSmooth
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.gen_castbar(self)
    if cfg.showPortraits then
      lib.gen_portrait(self)
    end
    lib.createBuffs(self)
    lib.createDebuffs(self)
    lib.gen_mirrorcb(self)
    lib.gen_ppstrings(self)
    lib.gen_Runes(self)
    lib.gen_TotemBar(self)
    lib.gen_InfoIcons(self)
    lib.gen_tempench(self)
    if cfg.playerauras then
      lib.createAuras(self)
    end
    lib.gen_combat_feedback(self)

    self.PvP = self:CreateTexture(nil, "OVERLAY")
	  self.PvP:SetPoint("TOPLEFT", self, -12,8)
	  self.PvP:SetSize(24,24)
    
    self.Resting = self.Power:CreateTexture(nil, "OVERLAY")
	  self.Resting:SetPoint("BOTTOMLEFT", self, -16, -12)
	  self.Resting:SetSize(24,24)
  end  
  
  --the target style
  local function CreateTargetStyle(self)
    self.width = cfg.Twidth
    self.height = cfg.Theight
    self.scale = cfg.Tscale
    self.mystyle = "target"
    lib.moveme(self)
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.Smooth = cfg.useSmooth
    --self.colors.smooth = {1,0,0, 0.7,0.41,0.44, 0.3,0.3,0.3}
		--self.colors.smooth = {1,0,0, 196/255,  30/255,  60/255, 0.3,0.3,0.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.gen_castbar(self)
    if cfg.showPortraits then
      lib.gen_portrait(self)
    end
    lib.createAuras(self)
    lib.gen_ppstrings(self)
    lib.gen_CPoints(self)
    if cfg.showfaketarget then
      lib.gen_faketarget(self)
    end
    lib.gen_combat_feedback(self)

    self.PvP = self:CreateTexture(nil, "OVERLAY")
	  self.PvP:SetPoint("TOPLEFT", self, -12,8)
	  self.PvP:SetSize(24,24)    
  end  
  
  --the tot style
  local function CreateToTStyle(self)
    self.width = cfg.PTTwidth
    self.height = cfg.PTTheight
    self.scale = cfg.PTTscale
    self.mystyle = "tot"
    lib.moveme(self)
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.bg.multiplier = 0.1
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.createBuffs(self)
    lib.createDebuffs(self)
  end 
  
  --the pet style
  local function CreatePetStyle(self)
    self.width = cfg.PTTwidth
    self.height = cfg.PTTheight
    self.scale = cfg.PTTscale
    self.mystyle = "pet"
    lib.moveme(self)
    self.disallowVehicleSwap = true
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.gen_castbar(self)
    lib.createBuffs(self)
    lib.createDebuffs(self)
  end  

  --the focus style
  local function CreateFocusStyle(self)
    self.width = cfg.Fwidth
    self.height = cfg.Fheight
    self.scale = cfg.Fscale
    self.mystyle = "focus"
    lib.moveme(self)
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.gen_castbar(self)
    lib.createAuras(self)
  end

  --party pets
  local function CreatePartyPetStyle(self)
    self.width = cfg.PABheight+cfg.PABheight/2.5+3
    self.height = self.width
    self.scale = 1
    self.mystyle = "partypet"
    genStyle(self)
    self.Range = {
      insideAlpha = 1,
      outsideAlpha = 0.6}
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
  end
  
  --the party style
  local function CreatePartyStyle(self, unit)
    if self:GetAttribute("unitsuffix") == "pet" then
      return CreatePartyPetStyle(self)
    end
    self.width = cfg.PABwidth
    self.height = cfg.PABheight
    self.scale = cfg.PABscale
    self.mystyle = "party"
    genStyle(self)
    self.Health.Smooth = cfg.useSmooth
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    self.Range = {
      insideAlpha = 1,
      outsideAlpha = 0.6}
    if cfg.usePortraits then
      lib.gen_portrait(self)
    end
    lib.createBuffs(self)
    lib.createDebuffs(self)
    lib.gen_InfoIcons(self)
    lib.gen_targeticon(self)
  end

  --arena frames
  local function CreateArenaStyle(self)
    self.width = cfg.PABwidth
    self.height = cfg.PABheight
    self.scale = cfg.PABscale
    self.mystyle = "arena"
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.createBuffs(self)
    lib.createDebuffs(self)
    lib.gen_ppstrings(self)
    lib.gen_castbar(self)
    lib.gen_arenatracker(self)
    lib.gen_targeticon(self)
  end

  --mini arena targets
  local function CreateArenaTargetStyle(self)
    self.width = cfg.PABheight+cfg.PABheight/2.5+3
    self.height = self.width
    self.scale = cfg.PABscale
    self.mystyle = "arenatarget"
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.colorHappiness = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
  end  
  
  --boss frames
  local function CreateBossStyle(self)
    self.width = cfg.PABwidth
    self.height = cfg.PABheight
    self.scale = cfg.PABscale
    self.mystyle = "boss"
    genStyle(self)
    self.Health.frequentUpdates = true
    self.Health.Smooth = cfg.useSmooth
    self.colors.smooth = {1,0,0, .7,.41,.44, .3,.3,.3}
    self.Health.colorSmooth = true
    self.Health.colorClass = cfg.useClassColorOnFullHealth
    self.Health.bg.multiplier = 0.1
    self.Power.frequentUpdates = true
    self.Power.colorPower = true
    self.Power.bg.multiplier = 0.3
    lib.gen_castbar(self)
  end  

  -----------------------------
  -- SPAWN UNITS
  -----------------------------
  oUF:RegisterStyle("monoPlayer", CreatePlayerStyle)
  oUF:SetActiveStyle("monoPlayer")
  oUF:Spawn("player", "oUF_monoPlayerFrame"):SetPoint(unpack(cfg.Ppos))

  oUF:RegisterStyle("monoTarget", CreateTargetStyle)
  oUF:SetActiveStyle("monoTarget")
  oUF:Spawn("target", "oUF_monoTargetFrame"):SetPoint(unpack(cfg.Tpos))

  if cfg.showtot then
    oUF:RegisterStyle("monoToT", CreateToTStyle)
    oUF:SetActiveStyle("monoToT")
    oUF:Spawn("targettarget", "oUF_mono_ToTFrame"):SetPoint(unpack(cfg.TTpos))
  end
  
  if cfg.showfocus then
    oUF:RegisterStyle("monoFocus", CreateFocusStyle)
    oUF:SetActiveStyle("monoFocus")
    oUF:Spawn("focus", "oUF_monoFocusFrame"):SetPoint(unpack(cfg.Fpos))
  else
    oUF:DisableBlizzard'focus'
  end
  
  if cfg.showpet then
    oUF:RegisterStyle("monoPet", CreatePetStyle)
    oUF:SetActiveStyle("monoPet")
    oUF:Spawn("pet", "oUF_monoPetFrame"):SetPoint(unpack(cfg.PEpos))
  end
  
  local visible = 'custom [group:party,nogroup:raid][@raid6,noexists,group:raid] show;hide'
  if cfg.showparty then
    oUF:RegisterStyle("monoParty", CreatePartyStyle)
    oUF:SetActiveStyle("monoParty")
    local party = oUF:SpawnHeader(nil,nil,
      visible,"showParty",true,
      --"solo,party","showPlayer", true,"showParty",true,
      "template","oUF_ppet",
      "yOffset",-cfg.PAspacing)
    party:SetPoint(unpack(cfg.PApos))
  else
    oUF:DisableBlizzard'party'
  end
  
  local gap = cfg.ABspacing
  if cfg.showarena then
    SetCVar("showArenaEnemyFrames", false)
    oUF:RegisterStyle("monoArena", CreateArenaStyle)
    oUF:SetActiveStyle("monoArena")
    local arena = {}
    local arenatarget = {}
    for i = 1, 5 do
      arena[i] = oUF:Spawn("arena"..i, "oUF_Arena"..i)
      if i == 1 then
        arena[i]:SetPoint(unpack(cfg.ARpos))
      else
        arena[i]:SetPoint("BOTTOMRIGHT", arena[i-1], "BOTTOMRIGHT", 0, gap)
      end
    end
    oUF:RegisterStyle("monoArenaTarget", CreateArenaTargetStyle)
    oUF:SetActiveStyle("monoArenaTarget")
    for i = 1, 5 do
      arenatarget[i] = oUF:Spawn("arena"..i.."target", "oUF_Arena"..i.."target"):SetPoint("TOPRIGHT",arena[i], "TOPLEFT", -4, 0)
    end
  end

  if cfg.showboss then
    oUF:RegisterStyle("monoBoss", CreateBossStyle)
    oUF:SetActiveStyle("monoBoss")
    local boss = {}
    for i = 1, MAX_BOSS_FRAMES do
      boss[i] = oUF:Spawn("boss"..i, "oUF_Boss"..i)
      if i == 1 then
        boss[i]:SetPoint(unpack(cfg.BOpos))
      else
        boss[i]:SetPoint("BOTTOMRIGHT", boss[i-1], "BOTTOMRIGHT", 0, gap)
      end
    end
  end