  local addon, ns = ...
  local cfg = ns.cfg
  local cast = ns.cast
  local lib = CreateFrame("Frame")  
  --[[TODO: 
    certain buffs hilights on arena frames (oUF_Auratracker)
  ]]
  -----------------------------
  -- local variables
  -----------------------------
  oUF.colors.power['MANA'] = {.3,.45,.65}
  oUF.colors.power['RAGE'] = {.7,.3,.3}
  oUF.colors.power['FOCUS'] = {.7,.45,.25}
  oUF.colors.power['ENERGY'] = {.65,.65,.35}
  oUF.colors.power['RUNIC_POWER'] = {.45,.45,.75}
  oUF.colors.power['AMMOSLOT'] = {.8,.6,0}
  oUF.colors.power['FUEL'] = {0,.55,.5}
  oUF.colors.power['POWER_TYPE_STEAM'] = {.55,.57,.61}
  oUF.colors.power['POWER_TYPE_PYRITE'] = {.6,.09,.17}
  oUF.colors.power['POWER_TYPE_HEAT'] = {.9,.45,.1}
  oUF.colors.power['POWER_TYPE_OOZE'] = {.1,.1,.9}
  oUF.colors.power['POWER_TYPE_BLOOD_POWER'] = {.9,.1,.1}
  local _, pType = UnitPowerType("player")
  local pcolor = oUF.colors.power[pType] or {.3,.45,.65}
  local class = select(2, UnitClass("player"))

  -----------------------------
  -- FUNCTIONS
  -----------------------------
  --fontstring func
  lib.gen_fontstring = function(f, name, size, outline)
    local fs = f:CreateFontString(nil, "OVERLAY")
    fs:SetFont(name, size, outline)
    fs:SetShadowColor(0,0,0,1)
--    fs:SetTextColor(1,1,1)
    return fs
  end  
  
  --status bar filling fix
  local fixStatusbar = function(b)
    b:GetStatusBarTexture():SetHorizTile(false)
    b:GetStatusBarTexture():SetVertTile(false)
  end

  --backdrop table
  local backdrop_tab = { 
    bgFile = cfg.backdrop_texture, 
    edgeFile = cfg.backdrop_edge_texture,
    tile = false, tileSize = 0, edgeSize = 5, 
    insets = {left = 5, right = 5, top = 5, bottom = 5,},}
  
  --backdrop func
  lib.gen_backdrop = function(f)
    f:SetBackdrop(backdrop_tab);
    f:SetBackdropColor(0,0,0,0.6)
    f:SetBackdropBorderColor(0,0,0,1)
  end
  
  --right click menu
  lib.menu = function(self)
    local unit = self.unit:sub(1, -2)
    local cunit = self.unit:gsub("(.)", string.upper, 1)
    if(cunit == 'Vehicle') then
      cunit = 'Player'
    end
    if unit == "party" then
      ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
    elseif(_G[cunit.."FrameDropDown"]) then
      ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
    end
  end
  
  --moveme func
  lib.moveme = function(f)
    if cfg.allow_frame_movement then
      f:SetMovable(true)
      f:SetUserPlaced(true)
      if not cfg.frames_locked then
        f:EnableMouse(true)
        f:RegisterForDrag("LeftButton","RightButton")
        f:SetScript("OnDragStart", function(self) if IsAltKeyDown() and IsShiftKeyDown() then self:StartMoving() end end)
        f:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
      end
    else
      f:IsUserPlaced(false)
    end  
  end 
  
  --init func
  lib.init = function(f)
    f:SetAttribute("initial-height", f.height)
    f:SetAttribute("initial-width", f.width)
    f:SetAttribute("initial-scale", f.scale)
    f:SetPoint("CENTER",UIParent,"CENTER",0,0)
    f.menu = lib.menu
    f:RegisterForClicks("AnyUp")
    f:SetAttribute("*type2", "menu")
    f:SetScript("OnEnter", UnitFrame_OnEnter)
    f:SetScript("OnLeave", UnitFrame_OnLeave)
  end
  
------ [Building frames]
  --gen healthbar func
  lib.gen_hpbar = function(f)
    --statusbar
    local s = CreateFrame("StatusBar", nil, f)
    s:SetStatusBarTexture(cfg.statusbar_texture)
    fixStatusbar(s)
    s:SetHeight(f.height)
    s:SetWidth(f.width)
    s:SetPoint("CENTER",0,0)
    s:SetAlpha(0.66)
    s:SetOrientation("HORIZONTAL") 
    --helper
    local h = CreateFrame("Frame", nil, s)
    h:SetFrameLevel(0)
    h:SetPoint("TOPLEFT",-4,4)
    h:SetPoint("BOTTOMRIGHT",4,-4)
    lib.gen_backdrop(h)
    --bg
    local b = s:CreateTexture(nil, "BACKGROUND")
    b:SetTexture(cfg.statusbar_texture)
    b:SetAllPoints(s)
    f.Health = s
    f.Health.bg = b
  end
  --gen hp strings func
  lib.gen_hpstrings = function(f, unit)
    --creating helper frame here so our font strings don't inherit healthbar parameters
    local h = CreateFrame("Frame", nil, f)
    h:SetAllPoints(f.Health)
    h:SetFrameLevel(15)
    local valsize
    if f.mystyle == "arenatarget" or f.mystyle == "partypet" then valsize = 11 else valsize = 13 end 
    local name = lib.gen_fontstring(h, cfg.font, 13, "THINOUTLINE")
    local hpval = lib.gen_fontstring(h, cfg.font, valsize, "THINOUTLINE")
    if f.mystyle == "target" or f.mystyle == "tot" then
      name:SetPoint("RIGHT", f.Health, "RIGHT",-3,0)
      hpval:SetPoint("LEFT", f.Health, "LEFT",3,0)
      name:SetJustifyH("RIGHT")
      name:SetPoint("LEFT", hpval, "RIGHT", 5, 0)
    elseif f.mystyle == "arenatarget" or f.mystyle == "partypet" then
      name:SetPoint("CENTER", f.Health, "CENTER",0,6)
      name:SetJustifyH("LEFT")
      hpval:SetPoint("CENTER", f.Health, "CENTER",0,-6)
    else
      name:SetPoint("LEFT", f.Health, "LEFT",3,0)
      hpval:SetPoint("RIGHT", f.Health, "RIGHT",-3,0)
      name:SetJustifyH("LEFT")
      name:SetPoint("RIGHT", hpval, "LEFT", -5, 0)
    end
    if f.mystyle == "arenatarget" or f.mystyle == "partypet" then
      f:Tag(name, '[mono:color][mono:shortname]')
      f:Tag(hpval, '[mono:hpraid]')
    else
      f:Tag(name, '[mono:color][mono:longname]')
      f:Tag(hpval, '[mono:hp]')
    end
  end
  
  --gen powerbar func
  lib.gen_ppbar = function(f)
    --statusbar
    local s = CreateFrame("StatusBar", nil, f)
    s:SetStatusBarTexture(cfg.statusbar_texture)
    fixStatusbar(s)
    s:SetHeight(f.height/2.5)
    s:SetWidth(f.width-1)
    s:SetPoint("TOP",f,"BOTTOM",0,-2)
    if f.mystyle == "partypet" or f.mystyle == "arenatarget" then
      s:Hide()
    end
    --helper
    local h = CreateFrame("Frame", nil, s)
    h:SetFrameLevel(0)
    h:SetPoint("TOPLEFT",-4,4)
    h:SetPoint("BOTTOMRIGHT",4,-4)
    lib.gen_backdrop(h)
    --bg
    local b = s:CreateTexture(nil, "BACKGROUND")
    b:SetTexture(cfg.statusbar_texture)
    b:SetAllPoints(s)
    if f.mystyle=="tot" or f.mystyle=="pet" then
      s:SetHeight(f.height/3)
    end
    f.Power = s
    f.Power.bg = b
  end
  --filling up powerbar with text strings
  lib.gen_ppstrings = function(f, unit)
    --helper frame
    local h = CreateFrame("Frame", nil, f)
    h:SetAllPoints(f.Power)
    h:SetFrameLevel(10)
    local fh
    if f.mystyle == "arena" then fh = 9 else fh = 11 end
    local pp = lib.gen_fontstring(h, cfg.font, fh, "THINOUTLINE")
    local info = lib.gen_fontstring(h, cfg.font, fh, "THINOUTLINE")
    if f.mystyle == "target" or f.mystyle == "tot" then
        info:SetPoint("RIGHT", f.Power, "RIGHT",-3,0)
        pp:SetPoint("LEFT", f.Power, "LEFT",3,0)
        info:SetJustifyH("RIGHT")
    else
        info:SetPoint("LEFT", f.Power, "LEFT",3,0)
        pp:SetPoint("RIGHT", f.Power, "RIGHT",-5,0)
        info:SetJustifyH("LEFT")
    end
  if class == "DRUID" then
    f:Tag(pp, '[mono:druidpower] [mono:pp]')
  else
    f:Tag(pp, '[mono:pp]')
  end
    f:Tag(info, '[mono:info]')
  end

  --3d portrait behind hp bar
  lib.gen_portrait = function(f)
    local p = CreateFrame("PlayerModel", nil, f)
    p:SetFrameStrata("BACKGROUND")
    p:SetWidth(f.width-2)
    p:SetHeight(f.height-2)
    p:SetPoint("TOP", f, "TOP", 0, -2)
    f.Portrait = p
  end

------ [Castbar, +mirror castbar too!]
  --gen castbar
  lib.gen_castbar = function(f)
    local s = CreateFrame("StatusBar", "oUF_monoCastbar"..f.mystyle, f)
    s:SetSize(f.width-(f.height/1.7+4),f.height/1.7)
    s:SetStatusBarTexture(cfg.statusbar_texture)
    s:SetStatusBarColor(pcolor[1], pcolor[2], pcolor[3],1)
    s:SetFrameLevel(9)
    --color
    s.CastingColor = {pcolor[1], pcolor[2], pcolor[3]}
    s.CompleteColor = {0.12, 0.86, 0.15}
    s.FailColor = {1.0, 0.09, 0}
    s.ChannelingColor = {pcolor[1], pcolor[2], pcolor[3]}
    --helper
    local h = CreateFrame("Frame", nil, s)
    h:SetFrameLevel(0)
    h:SetPoint("TOPLEFT",-4,4)
    h:SetPoint("BOTTOMRIGHT",4,-4)
    lib.gen_backdrop(h)
    --backdrop
    local b = s:CreateTexture(nil, "BACKGROUND")
    b:SetTexture(cfg.statusbar_texture)
    b:SetAllPoints(s)
    b:SetVertexColor(pcolor[1]*0.2,pcolor[2]*0.2,pcolor[3]*0.2,0.7)
    --spark
    sp = s:CreateTexture(nil, "OVERLAY")
    sp:SetBlendMode("ADD")
    sp:SetAlpha(0.5)
    sp:SetHeight(s:GetHeight()*2.5)
    --spell text
    local txt = lib.gen_fontstring(s, cfg.font, 11, "THINOUTLINE")
    txt:SetPoint("LEFT", 2, 0)
    txt:SetJustifyH("LEFT")
    --time
    local t = lib.gen_fontstring(s, cfg.font, 12, "THINOUTLINE")
    t:SetPoint("RIGHT", -2, 0)
    txt:SetPoint("RIGHT", t, "LEFT", -5, 0)
    --icon
    local i = s:CreateTexture(nil, "ARTWORK")
    i:SetSize(s:GetHeight()-2,s:GetHeight()-2)
    i:SetPoint("RIGHT", s, "LEFT", -4.5, 0)
    i:SetTexCoord(0.1, 0.9, 0.1, 0.9)
    --helper2 for icon
    local h2 = CreateFrame("Frame", nil, s)
    h2:SetFrameLevel(0)
    h2:SetPoint("TOPLEFT",i,"TOPLEFT",-5,5)
    h2:SetPoint("BOTTOMRIGHT",i,"BOTTOMRIGHT",5,-5)
    lib.gen_backdrop(h2)
    if f.mystyle == "player" then
      --latency only for player unit
      local z = s:CreateTexture(nil,"ADD")
      z:SetTexture(cfg.statusbar_texture)
      z:SetVertexColor(.8,.31,.45)
      z:SetPoint("TOPRIGHT")
      z:SetPoint("BOTTOMRIGHT")
      s.SafeZone = z
      --custom latency display
      local l = lib.gen_fontstring(s, cfg.font, 10, "THINOUTLINE")
      l:SetPoint("CENTER", -2, 17)
      l:SetJustifyH("RIGHT")
      s.Lag = l
      f:RegisterEvent("UNIT_SPELLCAST_SENT", cast.OnCastSent)
    end
    if f.mystyle == "focus" and cfg.focusCBuserplaced then
      lib.moveme(s)
      s:SetPoint(unpack(cfg.focusCBposition))
      s:SetSize(cfg.focusCBwidth,cfg.focusCBheight)
      i:SetSize(s:GetHeight()-2,s:GetHeight()-2)
      sp:SetHeight(s:GetHeight()*2.5)
    elseif f.mystyle == "pet" then
      s:SetPoint("BOTTOMRIGHT",f.Power,"BOTTOMRIGHT",0,0)
      s:SetScale(f:GetScale())
      s:SetSize(f.width-f.height/2,f.height*f.scale/2.5)
      i:SetPoint("RIGHT", s, "LEFT", -2, 0)
      h2:SetFrameLevel(9)
      b:Hide() txt:Hide() t:Hide() h:Hide()
    elseif f.mystyle == "arena" then
      s:SetSize(f.width-(f.height/1.4+4),f.height/1.4)
      s:SetPoint("TOPRIGHT",f.Power,"BOTTOMRIGHT",0,-4)
      i:SetPoint("RIGHT", s, "LEFT", -4, 0)
      i:SetSize(s:GetHeight()-2,s:GetHeight()-2)
    else
      s:SetPoint("TOPRIGHT",f.Power,"BOTTOMRIGHT",0,-4)
    end
    s.OnUpdate = cast.OnCastbarUpdate
    s.PostCastStart = cast.PostCastStart
    s.PostChannelStart = cast.PostCastStart
    s.PostCastStop = cast.PostCastStop
    s.PostChannelStop = cast.PostChannelStop
    s.PostCastFailed = cast.PostCastFailed
    s.PostCastInterrupted = cast.PostCastFailed

    f.Castbar = s
    f.Castbar.Text = txt
    f.Castbar.Time = t
    f.Castbar.Icon = i
    f.Castbar.Spark = sp
  end
  --gen Mirror Cast Bar
  lib.gen_mirrorcb = function(f)
    for _, bar in pairs({'MirrorTimer1','MirrorTimer2','MirrorTimer3',}) do   
      for i, region in pairs({_G[bar]:GetRegions()}) do
        if (region.GetTexture and region:GetTexture() == 'SolidTexture') then
          region:Hide()
        end
      end
      _G[bar..'Border']:Hide()
      _G[bar]:SetParent(UIParent)
      _G[bar]:SetScale(1)
      _G[bar]:SetHeight(16)
      _G[bar]:SetWidth(280)
      _G[bar]:SetBackdropColor(.1,.1,.1)
      _G[bar..'Background'] = _G[bar]:CreateTexture(bar..'Background', 'BACKGROUND', _G[bar])
      _G[bar..'Background']:SetTexture(cfg.statusbar_texture)
      _G[bar..'Background']:SetAllPoints(bar)
      _G[bar..'Background']:SetVertexColor(.15,.15,.15,.75)
      _G[bar..'Text']:SetFont(cfg.font, 14)
      _G[bar..'Text']:ClearAllPoints()
      _G[bar..'Text']:SetPoint('CENTER', MirrorTimer1StatusBar, 0, 1)
      _G[bar..'StatusBar']:SetAllPoints(_G[bar])
      --glowing borders
      local h = CreateFrame("Frame", nil, _G[bar])
      h:SetFrameLevel(0)
      h:SetPoint("TOPLEFT",-4,4)
      h:SetPoint("BOTTOMRIGHT",4,-4)
      lib.gen_backdrop(h)
    end
  end
  
------ [Auras, all of them!]
-- Creating our own timers with blackjack and hookers!
  lib.FormatTime = function(s)
    local day, hour, minute = 86400, 3600, 60
    if s >= day then
      return format("%dd", floor(s/day + 0.5)), s % day
    elseif s >= hour then
      return format("%dh", floor(s/hour + 0.5)), s % hour
    elseif s >= minute then
      if s <= minute * 5 then
        return format('%d:%02d', floor(s/60), s % minute), s - floor(s)
      end
      return format("%dm", floor(s/minute + 0.5)), s % minute
    elseif s >= minute / 12 then
      return floor(s + 0.5), (s * 100 - floor(s * 100))/100
    end
    return format("%.1f", s), (s * 100 - floor(s * 100))/100
  end
  lib.CreateAuraTimer = function(self,elapsed)
    if self.timeLeft then
      self.elapsed = (self.elapsed or 0) + elapsed
      local w = self:GetWidth()
      if self.elapsed >= 0.1 then
        if not self.first then
          self.timeLeft = self.timeLeft - self.elapsed
        else
          self.timeLeft = self.timeLeft - GetTime()
          self.first = false
        end
        if self.timeLeft > 0 and w > cfg.ATIconSizeThreshold then
          local time = lib.FormatTime(self.timeLeft)
          self.remaining:SetText(time)
          if self.timeLeft < 5 then
            self.remaining:SetTextColor(1, 0, 0)
          else
            self.remaining:SetTextColor(0.84, 0.75, 0.65)
          end
        else
          self.remaining:Hide()
          self:SetScript("OnUpdate", nil)
        end
        self.elapsed = 0
      end
    end
  end
  lib.PostUpdateIcon = function(self, unit, icon, index, offset)
  local _, _, _, _, _, duration, expirationTime, unitCaster, _ = UnitAura(unit, index, icon.filter)
    -- Debuff desaturation
    if unitCaster ~= 'player' and unitCaster ~= 'vehicle' and not UnitIsFriend('player', unit) and icon.debuff then
      icon.icon:SetDesaturated(true)
    else
      icon.icon:SetDesaturated(false)
    end
    -- Creating aura timers
      if duration and duration > 0 and cfg.auratimers then
        icon.remaining:Show()
      else
        icon.remaining:Hide()
      end
    if unit == 'player' then
      icon.duration = duration
      icon.timeLeft = expirationTime
      icon.first = true
      icon:SetScript("OnUpdate", lib.CreateAuraTimer)
    end
  end
  -- creating aura icons
  lib.PostCreateIcon = function(self, button)
    button.cd:SetReverse()
    button.cd.noOCC = true
    button.icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
    button.icon:SetDrawLayer("BACKGROUND")
    --count
    button.count:ClearAllPoints()
    button.count:SetJustifyH("RIGHT")
    button.count:SetPoint("BOTTOMRIGHT", 2, -2)
    button.count:SetTextColor(0.7,0.7,0.7)
    --helper
    local h = CreateFrame("Frame", nil, button)
    h:SetFrameLevel(0)
    h:SetPoint("TOPLEFT",-4,4)
    h:SetPoint("BOTTOMRIGHT",4,-4)
    lib.gen_backdrop(h)
    --another helper frame for our fontstring to overlap the cd frame
    local h2 = CreateFrame("Frame", nil, button)
    h2:SetAllPoints(button)
    h2:SetFrameLevel(10)
    button.remaining = lib.gen_fontstring(h2, cfg.font, cfg.ATSize, "THINOUTLINE")
    button.remaining:SetPoint("TOP", 0, -2)
    --overlay texture for debuff types display
    button.overlay:SetTexture(cfg.auratex)
    button.overlay:SetPoint("TOPLEFT", button, "TOPLEFT", -1.6, 1.6)
    button.overlay:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 1.6, -1.6)
    button.overlay:SetTexCoord(0.03, 0.96, 0.03, 0.96)
    button.overlay.Hide = function(self) self:SetVertexColor(0, 0, 0) end
  end

  --auras for certain frames
  lib.createAuras = function(f)
    a = CreateFrame('Frame', nil, f)
    a:SetPoint('BOTTOMLEFT', f, 'TOPLEFT', 1.5, 4)
    a['growth-x'] = 'RIGHT'
    a['growth-y'] = 'UP' 
    a.initialAnchor = 'BOTTOMLEFT'
    a.gap = true
    a.spacing = 6
    a.size = 20
    a.showDebuffType = true
    if f.mystyle=="target" or (f.mystyle=="player" and cfg.playerauras) then
      if f.mystyle=="player" and (class == "DEATHKNIGHT" or (class == "SHAMAN" and IsAddOnLoaded("oUF_TotemBar"))) then 
        -- making space for rune or totem bar
        a:SetPoint('BOTTOMLEFT', f, 'TOPLEFT', 1, 6+f.height/3)
      end
      a:SetHeight((a.size+a.spacing)*2)
      a:SetWidth((a.size+a.spacing)*9)
      a.numBuffs = 20 
      a.numDebuffs = 20
    elseif f.mystyle=="focus" then
      a.size = 20
      a.spacing = 4.5
      a:SetHeight((a.size+a.spacing)*2)
      a:SetWidth((a.size+a.spacing)*5)
      a.numBuffs = 10 
      a.numDebuffs = 10 
    end
    f.Auras = a
    a.PostCreateIcon = lib.PostCreateIcon
    a.PostUpdateIcon = lib.PostUpdateIcon
  end
  -- buffs
  lib.createBuffs = function(f)
    --Hiding Deafault Player's buff frames
    if cfg.stylebuffs then
      _G["BuffFrame"]:Hide()
      _G["BuffFrame"]:UnregisterAllEvents()
      _G["BuffFrame"]:SetScript("OnUpdate", nil)
    end
    b = CreateFrame("Frame", nil, f)
    b.initialAnchor = "TOPLEFT"
    b["growth-y"] = "DOWN"
    b.num = 5
    b.size = 18
    b.spacing = 4.5
    b:SetHeight((b.size+b.spacing)*2)
    b:SetWidth((b.size+b.spacing)*12)
    if f.mystyle=="player" and cfg.stylebuffs then
      b:SetPoint(unpack(cfg.BUFFpos))
      b.initialAnchor = "TOPRIGHT"
      b['growth-x'] = "LEFT"
      b.size = cfg.BFIsize
      b:SetWidth((b.size+b.spacing+1) * 15)
      b.num = 30
      b.spacing = 6
    elseif f.mystyle=="pet" then
      b.initialAnchor = "TOPRIGHT"
      b:SetPoint("TOPRIGHT", f, "TOPLEFT", -b.spacing, -1)
      b["growth-x"] = "LEFT"
    elseif f.mystyle=="tot" then
      b:SetPoint("TOPLEFT", f, "TOPRIGHT", b.spacing, -1)
    elseif f.mystyle=="arena" and not cfg.ArenaAuraTracker then
      b.showBuffType = true
      b:SetPoint("TOPLEFT", f, "TOPRIGHT", b.spacing, -1)
      b.num = 4
      b:SetWidth((b.size+b.spacing)*4)
    elseif f.mystyle=='party' then
      b:SetPoint("TOPLEFT", f.Power, "BOTTOMLEFT", 0, -b.spacing)
      b.num = 9
      b.spacing = 4
    end

    b.PostCreateIcon = lib.PostCreateIcon
    b.PostUpdateIcon = lib.PostUpdateIcon
    f.Buffs = b
  end
  --debuffs
  lib.createDebuffs = function(f)
    d = CreateFrame("Frame", nil, f)
    d.initialAnchor = "TOPRIGHT"
    d["growth-y"] = "DOWN"
    d.num = 5
    d.size = 18
    d.spacing = 4.5
    d:SetHeight((d.size+d.spacing)*2)
    d:SetWidth((d.size+d.spacing) * 5)
    d.showDebuffType = true
    if f.mystyle=="player" and cfg.stylebuffs then
      d:SetPoint(unpack(cfg.DEBUFFpos))
      d.size = cfg.BFIsize
      d:SetWidth((d.size+d.spacing) * 15)
      d['growth-x'] = "LEFT"
      d.num = 15
      d.spacing = 6
    elseif f.mystyle=="pet" then
      d:SetPoint("TOPLEFT", f, "TOPRIGHT", d.spacing, -1)
      d.initialAnchor = "TOPLEFT"
    elseif f.mystyle=="tot" then
      d:SetPoint("TOPRIGHT", f, "TOPLEFT", -d.spacing, -1)
      d["growth-x"] = "LEFT"
    elseif f.mystyle=="arena" and not cfg.ArenaAuraTracker then
      d.showDebuffType = false
      d.initialAnchor = "TOPLEFT"
      d:SetPoint("TOPLEFT", f, "TOPRIGHT", d.spacing, -d.size-d.spacing*2)
      d.num = 4
      d:SetWidth((d.size+d.spacing)*4)
    elseif f.mystyle=='party' then
      d.num = 8
      d:SetPoint("TOPRIGHT", f, "TOPLEFT", -d.spacing, -1)
      d["growth-x"] = "LEFT"
      d:SetWidth((d.size+d.spacing)*4)
    end
    d.PostCreateIcon = lib.PostCreateIcon
    d.PostUpdateIcon = lib.PostUpdateIcon
    f.Debuffs = d
  end

------ [Extra functionality]
  --gen DK runes
  lib.gen_Runes = function(f)
    if class ~= "DEATHKNIGHT" then return
    else
      local runeloadcolors = {
      [1] = {0.59, 0.31, 0.31},
      [2] = {0.59, 0.31, 0.31},
      [3] = {0.33, 0.51, 0.33},
      [4] = {0.33, 0.51, 0.33},
      [5] = {0.31, 0.45, 0.53},
      [6] = {0.31, 0.45, 0.53},}
      f.Runes = CreateFrame("Frame", nil, f)
      for i = 1, 6 do
        r = CreateFrame("StatusBar", f:GetName().."_Runes"..i, f)
        r:SetSize(f.width/6 - 2, f.height/3)
        if (i == 1) then
          r:SetPoint("BOTTOMLEFT", f, "TOPLEFT", 1, 3)
        else
          r:SetPoint("TOPLEFT", f.Runes[i-1], "TOPRIGHT", 2, 0)
        end
        r:SetStatusBarTexture(cfg.statusbar_texture)
        r:GetStatusBarTexture():SetHorizTile(false)
        r:SetStatusBarColor(unpack(runeloadcolors[i]))
        r.bd = r:CreateTexture(nil, "BORDER")
        r.bd:SetAllPoints()
        r.bd:SetTexture(cfg.statusbar_texture)
        r.bd:SetVertexColor(0.15, 0.15, 0.15)
        f.b = CreateFrame("Frame", nil, r)
        f.b:SetPoint("TOPLEFT", r, "TOPLEFT", -4, 4)
        f.b:SetPoint("BOTTOMRIGHT", r, "BOTTOMRIGHT", 4, -5)
        f.b:SetBackdrop(backdrop_tab)
        f.b:SetBackdropColor(0, 0, 0, 0)
        f.b:SetBackdropBorderColor(0,0,0,1)
        f.Runes[i] = r
      end
    end
  end
  --gen TotemBar for shamans
  lib.gen_TotemBar = function(f)
    if class ~= "SHAMAN" then return
    elseif IsAddOnLoaded("oUF_TotemBar") then
      f.TotemBar = {} 
      for i = 1, 4 do 
        tb = CreateFrame("StatusBar", nil, f) 
        tb:SetSize(f.width/4 - 2, f.height/3)
        tb:SetStatusBarTexture(cfg.statusbar_texture)
        tb:GetStatusBarTexture():SetHorizTile(false)
        tb:SetMinMaxValues(0, 1) 
        if (i == 1) then 
          tb:SetPoint("BOTTOMLEFT", f, "TOPLEFT", 1, 3)
        else 
          tb:SetPoint("TOPLEFT", f.TotemBar[i-1], "TOPRIGHT", 2, 0) 
        end 
        tb.bd = tb:CreateTexture(nil, "BORDER")
        tb.bd:SetAllPoints()
        tb.bd:SetTexture(cfg.statusbar_texture)
        tb.bd:SetVertexColor(0.15, 0.15, 0.15)
        tb.bg = CreateFrame("Frame", nil, tb)
        tb.bg:SetPoint("TOPLEFT", tb, "TOPLEFT", -4, 4)
        tb.bg:SetPoint("BOTTOMRIGHT", tb, "BOTTOMRIGHT", 4, -5)
        tb.bg:SetBackdrop(backdrop_tab)
        tb.bg:SetBackdropColor(0,0,0,0)
        tb.bg:SetBackdropBorderColor(0,0,0,1)
        f.TotemBar[i] = tb
      end 
    end
  end
  --gen combo points
  lib.CPUpdate = function(self, event, unit)
  if unit == "pet" then return end
  local cp = UnitExists("vehicle") and GetComboPoints("vehicle", "target") or GetComboPoints("player", "target")
  local cpoints = self.CPoints
  for i = 1, MAX_COMBO_POINTS do
    if i <= cp then
      cpoints[i]:Show()
    else
      cpoints[i]:Hide()
    end
  end
  if self.Auras then
    self.Auras:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 1.5, cp > 0 and cfg.Theight/3+8 or 4)
  end
  end
  lib.gen_CPoints = function(f)
    if f.mystyle == 'target' then
      f.CPoints = {}
      for i = 1, MAX_COMBO_POINTS do
        cp = CreateFrame("Frame",nil,f)
        cp:SetSize(f.width/5 - 2, f.height/3)
        if i == 1 then
          cp:SetPoint("BOTTOMLEFT", f, "TOPLEFT", 1, 3)
        else
          cp:SetPoint('TOPLEFT', f.CPoints[i-1], 'TOPRIGHT', 2, 0)
        end
        cp.bd = cp:CreateTexture(nil,"LOW")
        cp.bd:SetAllPoints(cp)
        cp.bd:SetTexture(cfg.statusbar_texture)
        cp.bg = CreateFrame("Frame", nil, cp)
        cp.bg:SetPoint("TOPLEFT", cp, "TOPLEFT", -4, 4)
        cp.bg:SetPoint("BOTTOMRIGHT", cp, "BOTTOMRIGHT", 4, -5)
        cp.bg:SetBackdrop(backdrop_tab)
        cp.bg:SetBackdropColor(0,0,0,0)
        cp.bg:SetBackdropBorderColor(0,0,0,1)
        f.CPoints[i] = cp
      end
      f.CPoints[1].bd:SetVertexColor(.3,.6,.3)
      f.CPoints[2].bd:SetVertexColor(.3,.6,.3)
      f.CPoints[3].bd:SetVertexColor(.6,.6,.3)
      f.CPoints[4].bd:SetVertexColor(.6,.6,.3)
      f.CPoints[5].bd:SetVertexColor(.7,.3,.3)
    end 
    f.CPoints.Update = lib.CPUpdate
  end
  --gen combat and LFD icons
  lib.gen_InfoIcons = function(f)
    local h = CreateFrame("Frame",nil,f)
    h:SetAllPoints(f.Health)
    h:SetFrameLevel(10)
    --combat icon
    if f.mystyle == 'player' then
      f.Combat = h:CreateTexture(nil, 'OVERLAY')
      f.Combat:SetSize(14,14)
      f.Combat:SetPoint('TOPRIGHT', -1, 7)
      f.Combat:SetTexture('Interface\\CharacterFrame\\UI-StateIcon')
      f.Combat:SetTexCoord(0.58, 0.90, 0.08, 0.41)
    end
    --LFDRole icon
    if f.mystyle =='party' then 
      f.LFDRole = f.Power:CreateTexture(nil, 'OVERLAY')
      f.LFDRole:SetSize(14,14)
      f.LFDRole:SetPoint('LEFT', 2, 0)
    end
    --Leader icon
    li = h:CreateTexture(nil, "OVERLAY")
    li:SetPoint("TOPLEFT", f, 0, 6)
    li:SetSize(12,12)
    f.Leader = li
    --Assist icon
    ai = h:CreateTexture(nil, "OVERLAY")
    ai:SetPoint("TOPLEFT", f, 0, 6)
    ai:SetSize(12,12)
    f.Assistant = ai
    --ML icon
    local ml = h:CreateTexture(nil, 'OVERLAY')
    ml:SetSize(12,12)
    ml:SetPoint('LEFT', f.Leader, 'RIGHT')
    f.MasterLooter = ml
  end
  --gen raid mark icons
  lib.gen_RaidMark = function(f)
    local h = CreateFrame("Frame", nil, f)
    h:SetAllPoints(f)
    h:SetFrameLevel(10)
    h:SetAlpha(cfg.RMalpha)
    local ri = h:CreateTexture(nil,'OVERLAY',h)
    ri:SetPoint("CENTER", f, "CENTER", 0, 0)
    ri:SetSize(cfg.RMsize, cfg.RMsize)
    f.RaidIcon = ri
  end
  
  --gen Temporary enchant frame
  lib.gen_tempench = function(f)
    if cfg.stylebuffs then
      local ench,i = {},0
      for i = 1,2 do
        ench[i] = _G["TempEnchant"..i]
        _G["TempEnchant"..i.."Border"]:Hide()
        --_G["TempEnchant"..i.."Duration"]:SetFont(cfg.font, 9, "THINOUTLINE")
        ench[i]:ClearAllPoints()
        ench[i]:SetWidth(30)
        ench[i]:SetHeight(30)
        ench[i]:Show()
        if i==1 then 
          ench[i]:SetPoint("TOPRIGHT", f, "TOPLEFT", -10, 0)
        elseif i==2 then
          ench[i]:SetPoint("RIGHT", ench[i-1], "LEFT", -10, 0)
        end
        --helper frame for nifty background
        local h = CreateFrame("Frame", nil, ench[i])
        h:SetFrameLevel(20)
        h:SetPoint("TOPLEFT",-3.5,3.5)
        h:SetPoint("BOTTOMRIGHT",3.5,-3.5)
        h:SetBackdrop(backdrop_tab);
        h:SetBackdropColor(0,0,0,0)
        h:SetBackdropBorderColor(0,0,0,1)
      end
    end
  end
  --gen hilight texture
  lib.gen_highlight = function(f)
    local OnEnter = function(f)
      UnitFrame_OnEnter(f)
      f.Highlight:Show()
    end
    local OnLeave = function(f)
      UnitFrame_OnLeave(f)
      f.Highlight:Hide()
    end
    f:SetScript("OnEnter", OnEnter)
    f:SetScript("OnLeave", OnLeave)
    local hl = f.Health:CreateTexture(nil, "OVERLAY")
    hl:SetAllPoints(f.Health)
    hl:SetTexture(cfg.backdrop_texture)
    hl:SetVertexColor(.5,.5,.5,.1)
    hl:SetBlendMode("ADD")
    hl:Hide()
    f.Highlight = hl
  end
  --gen trinket and AuraTracker icon for arena frames
  lib.UpdateAuraTracker = function(self, elapsed)
    if self.active then
      self.timeleft = self.timeleft - elapsed
      if self.timeleft <= 5 then
        self.text:SetTextColor(1, 0, 0)
      else
        self.text:SetTextColor(0.84, 0.75, 0.65)
      end
      if self.timeleft <= 0 then
        self.icon:SetTexture("")
        self.text:SetText("")
      end
    self.text:SetFormattedText("%.1f", self.timeleft)
    end
  end
  lib.gen_arenatracker = function(f)
    t = CreateFrame("Frame", nil, f)
    if cfg.ArenaAuraTracker then
      local tsize = cfg.PABheight+cfg.PABheight/2.5+2
      t:SetSize(tsize,tsize)
      t:SetPoint("TOPLEFT", f, "TOPRIGHT", 4,0)
    else
      t:SetSize(20,20)
      t:SetPoint("TOP", f, "BOTTOM", 1, 4)
    end
    t:SetFrameLevel(30)
    t:SetAlpha(0.8)
    t.trinketUseAnnounce = true
    at = CreateFrame("Frame", nil, f)
    at:SetSize(t:GetWidth(),t:GetHeight())
    at:SetPoint("CENTER", t, "CENTER")
    at:SetFrameStrata("HIGH")
    at.icon = at:CreateTexture(nil, "ARTWORK")
    at.icon:SetSize(t:GetWidth(),t:GetHeight())
    at.icon:SetPoint("CENTER", t, "CENTER")
    at.icon:SetTexCoord(0.07,0.93,0.07,0.93)
    at.bg = CreateFrame("Frame", nil, at)
    at.bg:SetPoint("TOPLEFT",-4,4)
    at.bg:SetPoint("BOTTOMRIGHT",4,-4)
    at.bg:SetBackdrop(backdrop_tab);
    at.bg:SetBackdropColor(0,0,0,0)
    at.bg:SetBackdropBorderColor(0,0,0,1)
    at.text = lib.gen_fontstring(at, cfg.font, cfg.AATTimerSize, "THINOUTLINE")
    at.text:SetPoint('CENTER', at, 0, 0)
    at:SetScript("OnUpdate", lib.UpdateAuraTracker)
    f.Trinket = t
    f.AuraTracker = at
  end
  --gen current target indicator
  lib.gen_targeticon = function(f)
    local h = CreateFrame("Frame", nil, f)
    h:SetAllPoints(f.Health)
    h:SetFrameLevel(10)
    local ti = lib.gen_fontstring(h, cfg.font, 12, "THINOUTLINE")
    ti:SetPoint("LEFT", f.Health, "BOTTOMLEFT",-5,0)
    ti:SetJustifyH("LEFT")
    f:Tag(ti, '[mono:targeticon]')
  end
  --gen fake target bars
  lib.gen_faketarget = function(f)
    local fhp = CreateFrame("frame","FakeHealthBar",UIParent) 
    fhp:SetAlpha(.6)
    fhp:SetSize(f.width,f.height)
    fhp:SetPoint("TOPLEFT",oUF_monoTargetFrame,"TOPLEFT",0,0)
    fhp.bg = fhp:CreateTexture(nil, "PARENT")
    fhp.bg:SetTexture(cfg.statusbar_texture)
    fhp.bg:ClearAllPoints()
    fhp.bg:SetAllPoints(fhp)
    fhp.bg:SetVertexColor(.3,.3,.3)
    local h = CreateFrame("Frame",nil,fhp)
    h:SetBackdrop(backdrop_tab)
    h:SetPoint("TOPLEFT",-3.5,3.5)
    h:SetPoint("BOTTOMRIGHT",3.5,-3.5)
    h:SetBackdropColor(0,0,0,0)
    h:SetBackdropBorderColor(0,0,0,0.5)

    local fpp = CreateFrame("frame","FakeManaBar",fhp)
    fpp:SetWidth(fhp:GetWidth())
    fpp:SetHeight(f.height/2.5)
    fpp:SetPoint("TOPLEFT",FakeHealthBar,"BOTTOMLEFT",0,-2)
    fpp.bg = fpp:CreateTexture(nil, "PARENT")
    fpp.bg:SetTexture(cfg.statusbar_texture)
    fpp.bg:ClearAllPoints()
    fpp.bg:SetAllPoints(fpp)
    fpp.bg:SetVertexColor(.30,.45,.65)
    local h2 = CreateFrame("Frame",nil,fpp)
    h2:SetBackdrop(backdrop_tab)
    h2:SetPoint("TOPLEFT",-3.5,3.5)
    h2:SetPoint("BOTTOMRIGHT",3.5,-3.5)
    h2:SetBackdropColor(0,0,0,0)
    h2:SetBackdropBorderColor(0,0,0,0.5)

    fhp:RegisterEvent('PLAYER_TARGET_CHANGED')
    fhp:SetScript('OnEvent', function(self)
      if UnitExists("target") then
        self:Hide()
      else
        self:Show()
      end
    end)
  end
  -- oUF_CombatFeedback
  lib.gen_combat_feedback = function(f)
    if IsAddOnLoaded("oUF_CombatFeedback") then
      local h = CreateFrame("Frame", nil, f.Health)
      h:SetAllPoints(f.Health)
      h:SetFrameLevel(30)
      local cfbt = lib.gen_fontstring(h, cfg.font, 18, "THINOUTLINE")
      cfbt:SetPoint("CENTER", f.Health, "BOTTOM", 0, -1)
      cfbt.maxAlpha = 0.75
      cfbt.ignoreEnergize = true
      f.CombatFeedbackText = cfbt
    end
  end
  
  --hand the lib to the namespace for further usage
  ns.lib = lib