  local addon, ns = ...
  local cfg = ns.cfg
  local lib = ns.lib
  local lib_raid = ns.lib_raid
  local f = CreateFrame("Frame")
  f:SetScript("OnEvent", function(self, event, ...) return self[event](self, ...) end)
  local uw8 = ((cfg.width+cfg.spacing)*5/8-cfg.spacing) -- calculating unit width for 8 goups raid
  -----------------------------
  -- STYLE FUNCTIONS
  -----------------------------
  
  local function genStyle(self)
    lib.init(self)
    lib_raid.gen_hpbar(self)
    lib_raid.gen_hpstrings(self)
    lib.gen_highlight(self)
    lib_raid.gen_elements(self)
    lib_raid.upd_elements(self)
  end 

  local function CreateRaidStyle(self)
    self.width = cfg.width
    self.height = cfg.height
    self.scale = 1
    self.mystyle = "raid"
    genStyle(self)
    lib_raid.gen_ppbar(self)
    self.Range = {
        insideAlpha = 1,
        outsideAlpha = 0.5}
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.bg.multiplier = 0.1
  end 
  
  local function CreateBigRaidStyle(self)
    self.width = uw8
    self.height = cfg.height
    self.scale = 1
    self.mystyle = "raid"
    genStyle(self)
    lib_raid.gen_ppbar(self)
    self.Range = {
        insideAlpha = 1,
        outsideAlpha = 0.5}
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.bg.multiplier = 0.1
  end 
  
  local function CreateMTStyle(self)
    self.width = (1+(cfg.MTsize)/10) * cfg.width
    self.height = cfg.height
    self.scale = 1
    self.mystyle = "mtframe"
    genStyle(self)
    self.Range = {
        insideAlpha = 1,
        outsideAlpha = 0.5}
    self.Health.frequentUpdates = true
    self.Health.colorDisconnected = true
    self.Health.bg.multiplier = 0.1
  end 
  -----------------------------
  -- SPAWN UNITS
  -----------------------------
if cfg.showraid then
  oUF:RegisterStyle("oUF_monoRaid", CreateRaidStyle)
  oUF:SetActiveStyle"oUF_monoRaid"
  local visible
  if cfg.soloON and cfg.partyON then
    visible = 'raid,party,solo'
  elseif cfg.soloON and not cfg.partyON then
    visible = 'raid,solo'
  elseif cfg.partyON and not cfg.soloON then
    visible = 'raid,party'
  elseif not cfg.partyON and not cfg.soloON and not cfg.raid5ON then
    visible = 'custom [@raid6,exists,group:raid] show;hide'
  else
    visible = 'raid'
  end
  -- spawn default number of raid groups
  raid = {}
  for i = 1, 5 do 
    local group = oUF:SpawnHeader('oUF_monoRaid'..i, nil, visible,
      'showPlayer', true,
      'showSolo', true,
      'showParty', cfg.partyON,
      'showRaid', true,
      'xoffset', cfg.spacing, 
      'yOffset', 0,
      'point', "LEFT",
      'groupFilter', tostring(i),
      'groupingOrder', '1,2,3,4,5,6,7,8',
      'groupBy', 'GROUP',
      'maxColumns', 5,
      'unitsPerColumn', 5,
      'columnSpacing', cfg.spacing,
      'columnAnchorPoint', "DOWN")
    if i == 1 then
      group:SetPoint(cfg.pos[1], cfg.pos[2], cfg.pos[3], cfg.pos[4], cfg.pos[5])
    else
      group:SetPoint("TOPLEFT", raid[i-1], "BOTTOMLEFT", 0, -cfg.spacing)
    end
    raid[i] = group
  end
  
  --resize raid frames on demand
  oUF:RegisterStyle("oUF_monoBigRaid", CreateBigRaidStyle)
  function Show5()
    StaticPopupDialogs["RESETGRID"] = {
    text = "To reset grid layout to 25 man you must reload your UI. Would you like to perform it now?",
    button1 = YES,
    button2 = NO,
    OnAccept = function()
      ReloadUI();
    end,
    timeout = 0,
    whileDead = 1,
    hideOnEscape = 1}
    StaticPopup_Show("RESETGRID") 
  end
  local function shrink() 
    for i=1,8 do 
      for j=1,5 do
        local t = _G['oUF_monoRaid'..i..'UnitButton'..j]
        if t then
          t:SetWidth(uw8)
        end
      end
    end
  end
  function Show8()
    for i=1,8 do 
      for j=1,5 do
        local t = _G['oUF_monoRaid'..i..'UnitButton'..j]
        if t then return
        end
      end
    end
    if not (InCombatLockdown()) then
      DEFAULT_CHAT_FRAME:AddMessage("oUF_mono: 40 man raid layout is now active")
      oUF:SetActiveStyle"oUF_monoBigRaid"
      shrink()
      for i = 6, 8 do
        local group = oUF:SpawnHeader('oUF_monoRaid'..i, nil, visible,
            'showPlayer', true,
            'showSolo', true,
            'showParty', cfg.partyON,
            'showRaid', true,
            'xoffset', 0, 
            'yOffset', -(cfg.spacing),
            'point', 'TOP',
            'groupFilter', tostring(i),
            'groupingOrder', '1,2,3,4,5,6,7,8',
            'groupBy', 'GROUP',
            'maxColumns', 8,
            'unitsPerColumn', 5,
            'columnSpacing', cfg.spacing,
            'columnAnchorPoint', growth)
        if(i == 6) then  
          group:SetPoint(cfg.pos[1], cfg.pos[2], cfg.pos[3], cfg.pos[4]+5*(cfg.spacing+uw8), cfg.pos[5])
        else
          group:SetPoint('TOPLEFT', raid[i-1], 'TOPRIGHT', cfg.spacing, 0)
        end
        raid[i] = group
        group:SetAttribute('initial-width',uw8)
      end
      local tf=CreateFrame"Frame"
      tf:RegisterEvent('RAID_ROSTER_UPDATE')
      tf:RegisterEvent('PARTY_LEADER_CHANGED')
      tf:RegisterEvent('PARTY_MEMBERS_CHANGED')
      tf:SetScript('OnEvent', function(self) 
        if(InCombatLockdown()) then
          self:RegisterEvent('PLAYER_REGEN_ENABLED')
        else
          self:UnregisterEvent('PLAYER_REGEN_ENABLED')
          shrink()
        end
      end)
    else
      DEFAULT_CHAT_FRAME:AddMessage("oUF_mono: ERROR! You can't do that in combat!")
    end
  end
  -- registering slash command for raid layout swap between 5 and 8 groups
  SlashCmdList["monoraid"] = function(...) f:Commands(...) end 
  SLASH_monoraid1 = "/ouf"
  function f:Commands(cmd) 
      local c,num = string.split(" ",cmd)
      if c == "8" then 
      Show8()
    elseif c == "5" then
      Show5()
    else
      DEFAULT_CHAT_FRAME:AddMessage("Type '/ouf 8' or '/ouf 5' to set your raid layout to show either 8 groups or 5.")
    end
  end
  
  -- spawn MT targets (with oRA3 support!)
  oUF:RegisterStyle("oUF_monoMT", CreateMTStyle)
  oUF:SetActiveStyle"oUF_monoMT"
  if cfg.MTframes then
    local tank = oUF:SpawnHeader('oUF_monoMT', nil, visible,
      "showRaid", true,
      "yOffset", -cfg.spacing,
      "template", "oUF_Mtt")
    tank:SetPoint(cfg.MTpos[1], cfg.MTpos[2], cfg.MTpos[3], cfg.MTpos[4], cfg.MTpos[5])
    if oRA3 then
      tank:SetAttribute("initial-unitWatch", true)
      tank:SetAttribute("nameList", table.concat(oRA3:GetSortedTanks(), ","))
      local tankhandler = {}
      function tankhandler:OnTanksUpdated(event, tanks) 
        tank:SetAttribute("nameList", table.concat(tanks, ","))
      end
      oRA3.RegisterCallback(tankhandler, "OnTanksUpdated")
    else
      tank:SetAttribute('groupFilter', 'MAINTANK')
    end
  end
end

